package com.spectus.inapp

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.util.Log
import com.android.billingclient.api.*

class IAPHelper {

    var context : Context? = null
    var productId: String = ""
    var billingClient: BillingClient? = null
    var skuProductDetails: SkuDetails? = null
    var storedProductDataList: ArrayList<SkuDetails> = ArrayList()
    var purchaseDataSuccess = PurchaseData()

    var purchaseCallBack : ((errorMessage: String?, purchaseObject : PurchaseData?, purchaseID : String?) -> Unit)? = null

    fun startInAppPurchaseWith(
        contextObj: Context,
        productIdentifier: String,
        callback: (errorMessage: String?, purchaseObject : PurchaseData?, purchaseID : String?) -> Unit) {
        print("==>> startInAppPurchaseWith")
        productId = productIdentifier
        context = contextObj
        purchaseCallBack = callback
        this.setUpBillingClient(contextObj)
    }

    fun setUpBillingClient(context : Context) {
        // do your stuff

        billingClient = BillingClient.newBuilder(context)
            .setListener(purchaseUpdateListener)
            .enablePendingPurchases()
            .build()
        this.startConnection(context)
    }

    fun startConnection(context: Context) {
        billingClient?.startConnection(object : BillingClientStateListener {
            override fun onBillingSetupFinished(billingResult: BillingResult) {
                if (billingResult.responseCode == BillingClient.BillingResponseCode.OK) {
                    // The BillingClient is ready. You can query purchases here.
                    Log.v("TAG_INAPP", "Setup Billing Done")
                    queryAvaliableProducts(context)
                }
            }

            override fun onBillingServiceDisconnected() {

            }
        })
    }

    fun queryAvaliableProducts(context: Context) {
        val skuList = listOf(productId)

        val params = SkuDetailsParams.newBuilder()
            .setSkusList(skuList)
            .setType(BillingClient.SkuType.INAPP)
            .build()
//        val params = SkuDetailsParams.newBuilder()
//        params.setSkusList(skuList).setType(BillingClient.SkuType.INAPP)

        billingClient?.querySkuDetailsAsync(params) { billingResult, skuDetailsList
            ->
            // Process the result.
            if (billingResult.responseCode == BillingClient.BillingResponseCode.OK && !skuDetailsList.isNullOrEmpty()) {
                storedProductDataList.clear()
                storedProductDataList.addAll(skuDetailsList)
                for (skuDetails in skuDetailsList) {
                    Log.v("TAG_INAPP", "skuDetailsList : ${skuDetailsList}")
                    //This list should contain the products added above

                    this.updateUI(skuDetails, context)
                }
            }
        }
    }

    fun updateUI(skuDetails: SkuDetails?, context: Context) {
        skuDetails?.let {
            this.skuProductDetails = it
        }

        skuProductDetails?.let {
            val billingFlowParams = BillingFlowParams.newBuilder()
                .setSkuDetails(it)
                .build()
            billingClient?.launchBillingFlow(context as Activity, billingFlowParams)?.responseCode
        } ?: noSKUMessage()

    }

    private fun noSKUMessage() {

    }

    val purchaseUpdateListener = PurchasesUpdatedListener { billingResult, purchases ->
        Log.v("TAG_INAPP", "billingResult responseCode : ${billingResult.responseCode}")
        var errorMsg = "";
        if (billingResult.responseCode == Constant.OK && purchases != null) {
            for (purchase in purchases) {
                handleConsumedPurchases(purchase)
            }
        } else if (billingResult.responseCode == Constant.USER_CANCELED) {
            storedErrorData("User cancel the process.")
        } else if (billingResult.responseCode == Constant.SERVICE_UNAVAILABLE) {
            storedErrorData("Service is unavailable.")
        } else if (billingResult.responseCode == Constant.BILLING_UNAVAILABLE) {
            storedErrorData("Billing is unavailable.")
        } else if (billingResult.responseCode == Constant.ITEM_UNAVAILABLE) {
            storedErrorData("Item is unavailable.")
        } else if (billingResult.responseCode == Constant.DEVELOPER_ERROR) {
            storedErrorData("Deleloper Error.")
        } else if (billingResult.responseCode == Constant.ERROR) {
            storedErrorData("Somthing went wrong.")
        } else if (billingResult.responseCode == Constant.ITEM_ALREADY_OWNED) {
            storedErrorData("Item already owned.")
        } else if (billingResult.responseCode == Constant.ITEM_NOT_OWNED) {
            storedErrorData("Somthing went wrong.")
        } else if (billingResult.responseCode == Constant.SERVICE_TIMEOUT) {
            storedErrorData("Serverl timeout error.")
        } else if (billingResult.responseCode == Constant.FEATURE_NOT_SUPPORTED) {
            storedErrorData("Feature not supported.")
        } else if (billingResult.responseCode == Constant.SERVICE_DISCONNECTED) {
            storedErrorData("Server is disconnected.")
        } else {
            // Handle any other error codes.
        }
    }

    private fun handleConsumedPurchases(purchase: Purchase) {
        Log.d("TAG_INAPP", "handleConsumablePurchasesAsync foreach it is $purchase")
        var purchaseData = PurchaseData()
        purchaseData.orderId = purchase.orderId
        purchaseData.packageName = purchase.packageName
        purchaseData.purchaseTime = purchase.purchaseTime
        purchaseData.purchaseState = purchase.purchaseState
        purchaseData.developerPayload = purchase.developerPayload
        purchaseData.purchaseToken = purchase.purchaseToken
        purchaseData.autoRenewing = purchase.isAutoRenewing
        purchaseData.error_message = ""

        purchaseDataSuccess = purchaseData


        val params =
            ConsumeParams.newBuilder().setPurchaseToken(purchase.purchaseToken).build()
        billingClient?.consumeAsync(params) { billingResult, purchaseToken ->
            when (billingResult.responseCode) {
                Constant.OK -> {
                    // Update the appropriate tables/databases to grant user the items
                    var orderID = purchaseData.orderId
                    purchaseCallBack!!(null, purchaseData, orderID)
                }
            }
        }
    }

    fun storedErrorData(errorMsg: String) {
        var purchaseData = PurchaseData()
        purchaseData.orderId = ""
        purchaseData.packageName = ""
        purchaseData.purchaseTime = 0
        purchaseData.purchaseState = 0
        purchaseData.developerPayload = ""
        purchaseData.purchaseToken = ""
        purchaseData.autoRenewing = false
        purchaseData.error_message = errorMsg

        purchaseDataSuccess = purchaseData

        purchaseCallBack!!(errorMsg, purchaseData, null)
    }
}

class PurchaseData {
    var orderId: String? = null
    var packageName: String? = null
    var productId: String? = null
    var purchaseTime: Long = 0
    var purchaseState = 0
    var developerPayload: String? = null
    var purchaseToken: String? = null
    var error_message: String? = null
    var autoRenewing = false
}

class Constant {

    companion object {

        // App Product ID from Play Console
        var PRODUCT_ID = "com.monthly.inapp"
        var PRODUCT_ID_2 = "data_value_product"

        /*
 * Result code of BillingClient*/
        //Success

        // Codes for Billiing Details
        var SERVICE_TIMEOUT = -3
        var FEATURE_NOT_SUPPORTED = -2
        var SERVICE_DISCONNECTED = -1
        var OK = 0
        var USER_CANCELED = 1
        var SERVICE_UNAVAILABLE = 2
        var BILLING_UNAVAILABLE = 3
        var ITEM_UNAVAILABLE = 4
        var DEVELOPER_ERROR = 5
        var ERROR = 6
        var ITEM_ALREADY_OWNED = 7
        var ITEM_NOT_OWNED = 8

    }

}